@@ -26,6 +26,7 @@ import logging
 import StringIO
 import re
 import shutil
+import signal
 from hashlib import md5
 import luigi
 import luigi.hdfs
@@ -143,6 +144,29 @@ def flatten(sequence):
             yield item
 
 
+class HadoopRunContext(object):
+    def __init__(self):
+        self.job_id = None
+
+    def __enter__(self):
+        self.__old_signal = signal.getsignal(signal.SIGTERM)
+        signal.signal(signal.SIGTERM, self.kill_job)
+        return self
+
+    def kill_job(self, captured_signal=None, stack_frame=None):
+        if self.job_id:
+            logger.info('Job interrupted, killing job %s', self.job_id)
+            subprocess.call(['mapred', 'job', '-kill', self.job_id])
+        if captured_signal is not None:
+            # adding 128 gives the exit code corresponding to a signal
+            sys.exit(128 + captured_signal)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type is KeyboardInterrupt:
+            self.kill_job()
+        signal.signal(signal.SIGTERM, self.__old_signal)
+
+
 class HadoopJobError(RuntimeError):
     def __init__(self, message, out=None, err=None):
         super(HadoopJobError, self).__init__(message, out, err)
@@ -164,7 +188,7 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
         Currently just for tracking the job ID in a configuration where no history is stored in the output directory by Hadoop.
         '''
         history_filename = configuration.get_config().get('core', 'history-filename', '')
-        if history_filename:
+        if history_filename and '-output' in arglist:
             output_dir = arglist[arglist.index('-output') + 1]
             f = luigi.hdfs.HdfsTarget(os.path.join(output_dir, history_filename)).open('w')
             f.write(json.dumps(history))
@@ -173,40 +197,35 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
     def track_process(arglist, tracking_url_callback, env=None):
         # Dump stdout to a temp file, poll stderr and log it
         temp_stdout = tempfile.TemporaryFile()
-        proc = subprocess.Popen(arglist, stdout=temp_stdout, stderr=subprocess.PIPE, env=env)
+        proc = subprocess.Popen(arglist, stdout=temp_stdout, stderr=subprocess.PIPE, env=env, close_fds=True)
 
         # We parse the output to try to find the tracking URL.
         # This URL is useful for fetching the logs of the job.
         tracking_url = None
         job_id = None
         err_lines = []
-        while True:
-            try:
-                if proc.poll() is not None:
-                    break
+
+        with HadoopRunContext() as hadoop_context:
+            while proc.poll() is None:
                 err_line = proc.stderr.readline()
-            except KeyboardInterrupt:
-                if job_id:
-                    logger.info('Job interrupted by user, killing job %s', job_id)
-                    kill_job(job_id)
-                raise
-            err_lines.append(err_line)
-            err_line = err_line.strip()
-            if err_line:
-                logger.info('%s', err_line)
-            if err_line.find('Tracking URL') != -1:
-                tracking_url = err_line.split('Tracking URL: ')[-1]
-                try:
-                    tracking_url_callback(tracking_url)
-                except Exception as e:
-                    logger.error("Error in tracking_url_callback, disabling! %s", e)
-                    tracking_url_callback = lambda x: None
-            if err_line.find('Running job') != -1:
-                # hadoop jar output
-                job_id = err_line.split('Running job: ')[-1]
-            if err_line.find('submitted hadoop job:') != -1:
-                # scalding output
-                job_id = err_line.split('submitted hadoop job: ')[-1]
+                err_lines.append(err_line)
+                err_line = err_line.strip()
+                if err_line:
+                    logger.info('%s', err_line)
+                if err_line.find('Tracking URL') != -1:
+                    tracking_url = err_line.split('Tracking URL: ')[-1]
+                    try:
+                        tracking_url_callback(tracking_url)
+                    except Exception as e:
+                        logger.error("Error in tracking_url_callback, disabling! %s", e)
+                        tracking_url_callback = lambda x: None
+                if err_line.find('Running job') != -1:
+                    # hadoop jar output
+                    job_id = err_line.split('Running job: ')[-1]
+                if err_line.find('submitted hadoop job:') != -1:
+                    # scalding output
+                    job_id = err_line.split('submitted hadoop job: ')[-1]
+                hadoop_context.job_id = job_id
 
         # Read the rest + stdout
         err = ''.join(err_lines + [err_line for err_line in proc.stderr])
@@ -239,10 +258,6 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
     track_process(arglist, tracking_url_callback, env)
 
 
-def kill_job(job_id):
-    subprocess.call(['mapred', 'job', '-kill', job_id])
-
-
 def fetch_task_failures(tracking_url):
     ''' Uses mechanize to fetch the actual task logs from the task tracker.
 
@@ -311,15 +326,26 @@ class HadoopJobRunner(JobRunner):
         if runner_path.endswith("pyc"):
             runner_path = runner_path[:-3] + "py"
 
-        base_tmp_dir = configuration.get_config().get('core', 'tmp-dir', '/tmp/luigi')
-        self.tmp_dir = os.path.join(base_tmp_dir, 'hadoop_job_%016x' % random.getrandbits(64))
+        base_tmp_dir = configuration.get_config().get('core', 'tmp-dir', None)
+        if base_tmp_dir:
+            warnings.warn("The core.tmp-dir configuration item is"\
+                          " deprecated, please use the TMPDIR"\
+                          " environment variable if you wish"\
+                          " to control where luigi.hadoop may"\
+                          " create temporary files and directories.")
+            self.tmp_dir = os.path.join(base_tmp_dir, 'hadoop_job_%016x' % random.getrandbits(64))
+            os.makedirs(self.tmp_dir)
+        else:
+            self.tmp_dir = tempfile.mkdtemp()
+
         logger.debug("Tmp dir: %s", self.tmp_dir)
-        os.makedirs(self.tmp_dir)
 
         # build arguments
-        map_cmd = 'python mrrunner.py map'
-        cmb_cmd = 'python mrrunner.py combiner'
-        red_cmd = 'python mrrunner.py reduce'
+        config = configuration.get_config()
+        python_executable = config.get('hadoop', 'python-executable', 'python')
+        map_cmd = '{0} mrrunner.py map'.format(python_executable)
+        cmb_cmd = '{0} mrrunner.py combiner'.format(python_executable)
+        red_cmd = '{0} mrrunner.py reduce'.format(python_executable)
 
         # replace output with a temporary work directory
         output_final = job.output().path
@@ -395,6 +421,7 @@ class HadoopJobRunner(JobRunner):
         self.finish()
 
     def finish(self):
+        # FIXME: check for isdir?
         if self.tmp_dir and os.path.exists(self.tmp_dir):
             logger.debug('Removing directory %s', self.tmp_dir)
             shutil.rmtree(self.tmp_dir)
