@@ -41,7 +41,7 @@ class HDFSCliError(Exception):
 
 
 def call_check(command):
-    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
     stdout, stderr = p.communicate()
     if p.returncode != 0:
         raise HDFSCliError(command, p.returncode, stdout, stderr)
@@ -86,17 +86,20 @@ def list_path(path):
 
 class HdfsClient(FileSystem):
     """This client uses Apache 2.x syntax for file system commands, which also matched CDH4"""
+
+    recursive_listdir_cmd = ['-ls', '-R']
+
     def exists(self, path):
         """ Use `hadoop fs -stat to check file existence
         """
 
         cmd = [load_hadoop_cmd(), 'fs', '-stat', path]
-        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
         stdout, stderr = p.communicate()
         if p.returncode == 0:
             return True
         else:
-            not_found_pattern = "^stat: `.*': No such file or directory$"
+            not_found_pattern = "^.*No such file or directory$"
             not_found_re = re.compile(not_found_pattern)
             for line in stderr.split('\n'):
                 if not_found_re.match(line):
@@ -178,19 +181,19 @@ class HdfsClient(FileSystem):
             path = "."  # default to current/home catalog
 
         if recursive:
-            cmd = [load_hadoop_cmd(), 'fs', '-ls', '-R', path]
+            cmd = [load_hadoop_cmd(), 'fs'] + self.recursive_listdir_cmd + [path]
         else:
             cmd = [load_hadoop_cmd(), 'fs', '-ls', path]
-        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-        lines = proc.stdout
+        lines = call_check(cmd).split('\n')
 
         for line in lines:
-            if line.startswith('Found'):
-                continue  # "hadoop fs -ls" outputs "Found %d items" as its first line
-            line = line.rstrip("\n")
-            if ignore_directories and line[0] == 'd':
+            if not line:
                 continue
-            if ignore_files and line[0] == '-':
+            elif line.startswith('Found'):
+                continue  # "hadoop fs -ls" outputs "Found %d items" as its first line
+            elif ignore_directories and line[0] == 'd':
+                continue
+            elif ignore_files and line[0] == '-':
                 continue
             data = line.split(' ')
 
@@ -447,9 +450,12 @@ class HdfsClientCdh3(HdfsClient):
 class HdfsClientApache1(HdfsClientCdh3):
     """This client uses Apache 1.x syntax for file system commands,
     which are similar to CDH3 except for the file existence check"""
+
+    recursive_listdir_cmd = ['-lsr']
+
     def exists(self, path):
         cmd = [load_hadoop_cmd(), 'fs', '-test', '-e', path]
-        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
         stdout, stderr = p.communicate()
         if p.returncode == 0:
             return True
@@ -498,10 +504,10 @@ class HdfsAtomicWritePipe(luigi.format.OutputPipeProcessWrapper):
         self.tmppath = tmppath(self.path)
         tmpdir = os.path.dirname(self.tmppath)
         if get_hdfs_syntax() == "cdh4":
-            if subprocess.Popen([load_hadoop_cmd(), 'fs', '-mkdir', '-p', tmpdir]).wait():
+            if subprocess.Popen([load_hadoop_cmd(), 'fs', '-mkdir', '-p', tmpdir], close_fds=True).wait():
                 raise RuntimeError("Could not create directory: %s" % tmpdir)
         else:
-            if not exists(tmpdir) and subprocess.Popen([load_hadoop_cmd(), 'fs', '-mkdir', tmpdir]).wait():
+            if not exists(tmpdir) and subprocess.Popen([load_hadoop_cmd(), 'fs', '-mkdir', tmpdir], close_fds=True).wait():
                 raise RuntimeError("Could not create directory: %s" % tmpdir)
         super(HdfsAtomicWritePipe, self).__init__([load_hadoop_cmd(), 'fs', '-put', '-', self.tmppath])
 
