@@ -36,6 +36,8 @@ UPSTREAM_MISSING_INPUT = 'UPSTREAM_MISSING_INPUT'
 UPSTREAM_FAILED = 'UPSTREAM_FAILED'
 
 UPSTREAM_SEVERITY_ORDER = ('', UPSTREAM_RUNNING, UPSTREAM_MISSING_INPUT, UPSTREAM_FAILED)
+UPSTREAM_SEVERITY_KEY = lambda st: UPSTREAM_SEVERITY_ORDER.index(st)
+STATUS_TO_UPSTREAM_MAP = {FAILED: UPSTREAM_FAILED, RUNNING: UPSTREAM_RUNNING, PENDING: UPSTREAM_MISSING_INPUT}
 
 
 class Task(object):
@@ -63,15 +65,17 @@ class CentralPlannerScheduler(Scheduler):
     Can be run locally or on a server (using RemoteScheduler + server.Server).
     '''
 
-    def __init__(self, retry_delay=900.0, remove_delay=600.0, worker_disconnect_delay=60.0, task_history=None):
+    def __init__(self, retry_delay=900.0, remove_delay=600.0, worker_disconnect_delay=60.0,
+                 state_path='/var/lib/luigi-server/state.pickle', task_history=None):
         '''
         (all arguments are in seconds)
         Keyword Arguments:
         retry_delay -- How long after a Task fails to try it again, or -1 to never retry
         remove_delay -- How long after a Task finishes to remove it from the scheduler
+        state_path -- Path to state file (tasks and active workers)
         worker_disconnect_delay -- If a worker hasn't communicated for this long, remove it from active workers
         '''
-        self._state_path = '/var/lib/luigi-server/state.pickle'
+        self._state_path = state_path
         self._tasks = {}
         self._retry_delay = retry_delay
         self._remove_delay = remove_delay
@@ -193,11 +197,15 @@ class CentralPlannerScheduler(Scheduler):
         best_t = float('inf')
         best_task = None
         locally_pending_tasks = 0
+        running_tasks = []
 
         for task_id, task in self._tasks.iteritems():
             if worker not in task.workers:
                 continue
 
+            if task.status == RUNNING:
+                running_tasks.append({'task_id': task_id, 'worker': task.worker_running})
+
             if task.status != PENDING:
                 continue
 
@@ -220,47 +228,44 @@ class CentralPlannerScheduler(Scheduler):
             t.worker_running = worker
             self._update_task_history(best_task, RUNNING, host=host)
 
-        return locally_pending_tasks, best_task
+        return {'n_pending_tasks': locally_pending_tasks,
+                'task_id': best_task,
+                'running_tasks': running_tasks}
 
     def ping(self, worker):
         self.update(worker)
 
-    def _upstream_status(self, task, upstream_status_table):
-        def get_upstream_status(task, upstream_status_table):
-            if task.status != PENDING:
-                return ''
-            if not task.deps:
-                return UPSTREAM_MISSING_INPUT
-            status = ''
-            status_key = lambda st: UPSTREAM_SEVERITY_ORDER.index(st)
-            for dep_id in task.deps:
+    def _upstream_status(self, task_id, upstream_status_table):
+        if task_id in upstream_status_table:
+            return upstream_status_table[task_id]
+        elif task_id in self._tasks:
+            task_stack = [task_id]
+
+            while task_stack:
+                dep_id = task_stack.pop()
                 if dep_id in self._tasks:
                     dep = self._tasks[dep_id]
-                    if dep.status == FAILED:
-                        return UPSTREAM_FAILED
-                    if dep.status == RUNNING:
-                        status = max(status, UPSTREAM_RUNNING, key=status_key)
-                    elif dep.status == PENDING:
-                        status = max(status, self._upstream_status(dep, upstream_status_table), key=status_key)
-                    if status == UPSTREAM_FAILED:
-                        return UPSTREAM_FAILED
-            return status
+                    if dep_id not in upstream_status_table:
+                        if dep.status == PENDING and dep.deps:
+                            task_stack = task_stack + [dep_id] + list(dep.deps)
+                            upstream_status_table[dep_id] = ''  # will be updated postorder
+                        else:
+                            dep_status = STATUS_TO_UPSTREAM_MAP.get(dep.status, '')
+                            upstream_status_table[dep_id] = dep_status
+                    elif upstream_status_table[dep_id] == '' and dep.deps:
+                        # This is the postorder update step when we set the
+                        # status based on the previously calculated child elements
+                        upstream_status = [upstream_status_table.get(id, '') for id in dep.deps]
+                        upstream_status.append('')  # to handle empty list
+                        status = max(upstream_status, key=UPSTREAM_SEVERITY_KEY)
+                        upstream_status_table[dep_id] = status
+            return upstream_status_table[dep_id]
 
-        if task in upstream_status_table:
-            return upstream_status_table[task]
-        else:
-            task_status = get_upstream_status(task, upstream_status_table)
-            upstream_status_table[task] = task_status
-            return task_status
-
-    def _serialize_task(self, task_id, upstream_status_table):
+    def _serialize_task(self, task_id):
         task = self._tasks[task_id]
-        upstream_status = self._upstream_status(task, upstream_status_table)
-
         return {
             'deps': list(task.deps),
             'status': task.status,
-            'upstream_status': upstream_status,
             'workers': list(task.workers),
             'start_time': task.time,
             'params': self._get_task_params(task_id),
@@ -287,16 +292,15 @@ class CentralPlannerScheduler(Scheduler):
     def graph(self):
         self.prune()
         serialized = {}
-        upstream_status_table = {}
         for task_id, task in self._tasks.iteritems():
-            serialized[task_id] = self._serialize_task(task_id, upstream_status_table)
+            serialized[task_id] = self._serialize_task(task_id)
         return serialized
 
     def _recurse_deps(self, task_id, serialized):
         if task_id not in serialized:
             task = self._tasks.get(task_id)
             if task is None:
-                logger.warn('Missing task for id [%s]' % task_id)
+                logger.warn('Missing task for id [%s]', task_id)
                 serialized[task_id] = {
                     'deps': [],
                     'status': UNKNOWN,
@@ -306,8 +310,7 @@ class CentralPlannerScheduler(Scheduler):
                     'name': self._get_task_name(task_id)
                 }
             else:
-                upstream_status_table = {}
-                serialized[task_id] = self._serialize_task(task_id, upstream_status_table)
+                serialized[task_id] = self._serialize_task(task_id)
                 for dep in task.deps:
                     self._recurse_deps(dep, serialized)
 
@@ -322,11 +325,12 @@ class CentralPlannerScheduler(Scheduler):
         ''' query for a subset of tasks by status '''
         self.prune()
         result = {}
-        upstream_status_table = {}
+        upstream_status_table = {}  # used to memoize upstream status
         for task_id, task in self._tasks.iteritems():
             if not status or task.status == status:
-                serialized = self._serialize_task(task_id, upstream_status_table)
-                if task.status != PENDING or not upstream_status or upstream_status == serialized['upstream_status']:
+                if (task.status != PENDING or not upstream_status or
+                    upstream_status == self._upstream_status(task_id, upstream_status_table)):
+                    serialized = self._serialize_task(task_id)
                     result[task_id] = serialized
         return result
 
