@@ -138,6 +138,7 @@ class Register(abc.ABCMeta):
         return global_params.iteritems()
 
 
+
 class Task(object):
     __metaclass__ = Register
 
@@ -149,6 +150,31 @@ class Task(object):
 
     `Task._parameters` - list of (parameter_name, parameter) tuples for this task class
     """
+    _event_callbacks = {}
+
+    @classmethod
+    def event_handler(cls, event):
+        """ Decorator for adding event handlers """
+        def wrapped(callback):
+            cls._event_callbacks.setdefault(cls, {}).setdefault(event, set()).add(callback)
+            return callback
+        return wrapped
+
+    def trigger_event(self, event, *args, **kwargs):
+        """ Trigger that calls all of the specified events associated with this class.
+        """
+        for event_class, event_callbacks in self._event_callbacks.iteritems():
+            if not isinstance(self, event_class):
+                continue
+            for callback in event_callbacks.get(event, []):
+                try:
+                    # callbacks are protected
+                    callback(*args, **kwargs)
+                except KeyboardInterrupt:
+                    return
+                except:
+                    logger.exception("Error in event callback for %r", event)
+                    pass
 
     @property
     def task_family(self):
@@ -350,7 +376,6 @@ class Task(object):
         This method gets called when :py:meth:`run` completes without raising any exceptions.
         The returned value is json encoded and sent to the scheduler as the `expl` argument.
         Default behavior is to send an None value"""
-        return None
 
 
 def externalize(task):
